/*
 * ============================================================================
 * GNU Lesser General Public License
 * ============================================================================
 *
 * Square Brackets - Java Array Framework.
 * Copyright (C) 2013  Beatgrid Media B.V.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * Beatgrid Media B.V.
 * Sisalbaan 5-A
 * 2352 AZ Leiderdorp
 * The Netherlands
 *
 * info@squarebrackets.org
 * http://squarebrackets.org
 */
package org.squarebrackets;

import java.nio.Buffer;

/**
 * Interface for getting and setting array characteristics.
 *
 * <p>The methods of this interface throw an {@code IllegalStateException} if invoked outside scope of
 * {@link MutableArray#doAction doAction}.</p>
 *
 * @author Leon van Zantvoort
 */
public interface ArrayContext<E> extends ArrayCharacteristics<E>, MutableArrayCharacteristics<E> {

    /**
     * Returns the backing array (optional operation). Modifications to the backing array are reflected by the mutable
     * array.
     *
     * <p>The resulting object must be cast to the primitive or object type of the implementing class.</p>
     *
     * @return backing array.
     * @throws UnsupportedOperationException if the array on which {@code doAction} is invoked does not expose backing
     * array.
     */
    Object array();

    /**
     * Returns the backing buffer (optional operation). Modifications to the backing buffer are reflected by the mutable
     * array. The buffer is the actual backing buffer instance, giving the caller full control over the buffer's
     * position, limit and mark values. Changes to these values, directly, or in directly (i.e., {@code get} or
     * {@code put}) are reflected by {@code this} array. It is therefore advised to surround buffer operations with
     * {@code mark()} and {@code reset()}, unless such changes are intentional.
     *
     * @return backing buffer.
     * @throws UnsupportedOperationException if the array on which {@code doAction} is invoked does not expose backing
     * buffer.
     */
    Buffer buffer();
}
