/*
 * ============================================================================
 * GNU Lesser General Public License
 * ============================================================================
 *
 * Square Brackets - Java Array Framework.
 * Copyright (C) 2013  Beatgrid Media B.V.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * Beatgrid Media B.V.
 * Sisalbaan 5-A
 * 2352 AZ Leiderdorp
 * The Netherlands
 *
 * info@squarebrackets.org
 * http://squarebrackets.org
 */
package org.squarebrackets;

import org.squarebrackets.function.BooleanConsumer;
import org.squarebrackets.function.ByteConsumer;
import org.squarebrackets.function.CharConsumer;
import org.squarebrackets.function.FloatConsumer;
import org.squarebrackets.function.ShortConsumer;

import java.util.ListIterator;
import java.util.PrimitiveIterator;
import java.util.NoSuchElementException;
import java.util.function.Consumer;
import java.util.function.DoubleConsumer;
import java.util.function.IntConsumer;
import java.util.function.LongConsumer;

/**
 * Array iterator.
 *
 * @author Leon van Zantvoort
 */
public interface ArrayIterator<E> extends ListIterator<E> {

    /**
     * An array iterator specialized for {@code boolean} values.
     */
    public static interface OfBoolean extends ArrayIterator<Boolean> {

        /**
         * Returns the next {@code boolean} element in the iteration.
         *
         * @return the next {@code boolean} element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        boolean nextBoolean();

        /**
         * Returns the previous {@code boolean} element in the iteration.
         *
         * @return the previous {@code boolean} element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous element.
         */
        boolean previousBoolean();

        /**
         * Replaces the last element returned by {@link #nextBoolean} or {@link #previousBoolean} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addBoolean} have been
         * called after the last call to {@code nextInt} or {@code previousInt}.
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setInt} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         */
        void setBoolean(boolean value);

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #nextBoolean}, if any, and after the element that would be returned by
         * {@link #previousBoolean}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        void addBoolean(boolean value);

        /**
         * Returns the next element in the iteration.
         *
         * <p>Object counterpart of {@link #nextBoolean} method. It is advised to use {@link #nextBoolean} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        @Override
        default Boolean next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return nextBoolean();
        }

        /**
         * Returns the previous element in the iteration.
         *
         * <p>Object counterpart of {@link #previousBoolean} method. It is advised to use {@link #previousBoolean} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous elements.
         */
        @Override
        default Boolean previous() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return previousBoolean();
        }

        /**
         * Replaces the last element returned by {@link #nextBoolean} or {@link #previousBoolean} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addBoolean} have been
         * called after the last call to {@code next} or {@code previousInt}.
         *
         * <p>Object counterpart of {@link #setBoolean} method. It is advised to use {@link #setBoolean} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setBoolean} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextBoolean} nor {@code previousBoolean} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         * @throws NullPointerException if value is {@code null}.
         */
        @Override
        default void set(Boolean value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            setBoolean(value);
        }

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #next}, if any, and after the element that would be returned by
         * {@link #previousBoolean}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * <p>Object counterpart of {@link #addBoolean} method. It is advised to use {@link #addBoolean} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        @Override
        default void add(Boolean value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            addBoolean(value);
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextBoolean());
         * }</pre>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        default void forEachRemaining(BooleanConsumer action) {
            while (hasNext()) {
                action.accept(nextBoolean());
            }
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextBoolean());
         * }</pre>
         *
         * <p>Object counterpart of {@link #addBoolean} method. It is advised to use {@link #addBoolean} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        @Override
        default void forEachRemaining(Consumer<? super Boolean> action) {
            if (action instanceof BooleanConsumer) {
                forEachRemaining((BooleanConsumer) action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(getClass());
                }
                forEachRemaining((BooleanConsumer) action::accept);
            }
        }
    }

    /**
     * An array iterator specialized for {@code byte} values.
     */
    public static interface OfByte extends ArrayIterator<Byte> {

        /**
         * Returns the next {@code byte} element in the iteration.
         *
         * @return the next {@code byte} element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        byte nextByte();

        /**
         * Returns the previous {@code byte} element in the iteration.
         *
         * @return the previous {@code byte} element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous element.
         */
        byte previousByte();

        /**
         * Replaces the last element returned by {@link #nextByte} or {@link #previousByte} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addByte} have been
         * called after the last call to {@code nextInt} or {@code previousInt}.
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setInt} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         */
        void setByte(byte value);

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #nextByte}, if any, and after the element that would be returned by
         * {@link #previousByte}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        void addByte(byte value);

        /**
         * Returns the next element in the iteration.
         *
         * <p>Object counterpart of {@link #nextByte} method. It is advised to use {@link #nextByte} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        @Override
        default Byte next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return nextByte();
        }

        /**
         * Returns the previous element in the iteration.
         *
         * <p>Object counterpart of {@link #previousByte} method. It is advised to use {@link #previousByte} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous elements.
         */
        @Override
        default Byte previous() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return previousByte();
        }

        /**
         * Replaces the last element returned by {@link #nextByte} or {@link #previousByte} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addByte} have been
         * called after the last call to {@code next} or {@code previousInt}.
         *
         * <p>Object counterpart of {@link #setByte} method. It is advised to use {@link #setByte} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setByte} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextByte} nor {@code previousByte} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         * @throws NullPointerException if value is {@code null}.
         */
        @Override
        default void set(Byte value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            setByte(value);
        }

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #next}, if any, and after the element that would be returned by
         * {@link #previousByte}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * <p>Object counterpart of {@link #addByte} method. It is advised to use {@link #addByte} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        @Override
        default void add(Byte value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            addByte(value);
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextByte());
         * }</pre>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        default void forEachRemaining(ByteConsumer action) {
            while (hasNext()) {
                action.accept(nextByte());
            }
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextByte());
         * }</pre>
         *
         * <p>Object counterpart of {@link #addByte} method. It is advised to use {@link #addByte} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        @Override
        default void forEachRemaining(Consumer<? super Byte> action) {
            if (action instanceof ByteConsumer) {
                forEachRemaining((ByteConsumer) action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(getClass());
                }
                forEachRemaining((ByteConsumer) action::accept);
            }
        }
    }

    /**
     * An array iterator specialized for {@code short} values.
     */
    public static interface OfShort extends ArrayIterator<Short> {

        /**
         * Returns the next {@code short} element in the iteration.
         *
         * @return the next {@code short} element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        short nextShort();

        /**
         * Returns the previous {@code short} element in the iteration.
         *
         * @return the previous {@code short} element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous element.
         */
        short previousShort();

        /**
         * Replaces the last element returned by {@link #nextShort} or {@link #previousShort} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addShort} have been
         * called after the last call to {@code nextInt} or {@code previousInt}.
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setInt} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         */
        void setShort(short value);

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #nextShort}, if any, and after the element that would be returned by
         * {@link #previousShort}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        void addShort(short value);

        /**
         * Returns the next element in the iteration.
         *
         * <p>Object counterpart of {@link #nextShort} method. It is advised to use {@link #nextShort} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        @Override
        default Short next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return nextShort();
        }

        /**
         * Returns the previous element in the iteration.
         *
         * <p>Object counterpart of {@link #previousShort} method. It is advised to use {@link #previousShort} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous elements.
         */
        @Override
        default Short previous() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return previousShort();
        }

        /**
         * Replaces the last element returned by {@link #nextShort} or {@link #previousShort} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addShort} have been
         * called after the last call to {@code next} or {@code previousInt}.
         *
         * <p>Object counterpart of {@link #setShort} method. It is advised to use {@link #setShort} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setShort} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextShort} nor {@code previousShort} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         * @throws NullPointerException if value is {@code null}.
         */
        @Override
        default void set(Short value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            setShort(value);
        }

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #next}, if any, and after the element that would be returned by
         * {@link #previousShort}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * <p>Object counterpart of {@link #addShort} method. It is advised to use {@link #addShort} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        @Override
        default void add(Short value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            addShort(value);
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextShort());
         * }</pre>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        default void forEachRemaining(ShortConsumer action) {
            while (hasNext()) {
                action.accept(nextShort());
            }
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextShort());
         * }</pre>
         *
         * <p>Object counterpart of {@link #addShort} method. It is advised to use {@link #addShort} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        @Override
        default void forEachRemaining(Consumer<? super Short> action) {
            if (action instanceof ShortConsumer) {
                forEachRemaining((ShortConsumer) action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(getClass());
                }
                forEachRemaining((ShortConsumer) action::accept);
            }
        }
    }

    /**
     * An array iterator specialized for {@code char} values.
     */
    public static interface OfChar extends ArrayIterator<Character> {

        /**
         * Returns the next {@code char} element in the iteration.
         *
         * @return the next {@code char} element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        char nextChar();

        /**
         * Returns the previous {@code char} element in the iteration.
         *
         * @return the previous {@code char} element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous element.
         */
        char previousChar();

        /**
         * Replaces the last element returned by {@link #nextChar} or {@link #previousChar} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addChar} have been
         * called after the last call to {@code nextInt} or {@code previousInt}.
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setInt} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         */
        void setChar(char value);

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #nextChar}, if any, and after the element that would be returned by
         * {@link #previousChar}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        void addChar(char value);

        /**
         * Returns the next element in the iteration.
         *
         * <p>Object counterpart of {@link #nextChar} method. It is advised to use {@link #nextChar} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        @Override
        default Character next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return nextChar();
        }

        /**
         * Returns the previous element in the iteration.
         *
         * <p>Object counterpart of {@link #previousChar} method. It is advised to use {@link #previousChar} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous elements.
         */
        @Override
        default Character previous() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return previousChar();
        }

        /**
         * Replaces the last element returned by {@link #nextChar} or {@link #previousChar} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addChar} have been
         * called after the last call to {@code next} or {@code previousInt}.
         *
         * <p>Object counterpart of {@link #setChar} method. It is advised to use {@link #setChar} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setChar} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextChar} nor {@code previousChar} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         * @throws NullPointerException if value is {@code null}.
         */
        @Override
        default void set(Character value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            setChar(value);
        }

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #next}, if any, and after the element that would be returned by
         * {@link #previousChar}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * <p>Object counterpart of {@link #addChar} method. It is advised to use {@link #addChar} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        @Override
        default void add(Character value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            addChar(value);
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextChar());
         * }</pre>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        default void forEachRemaining(CharConsumer action) {
            while (hasNext()) {
                action.accept(nextChar());
            }
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextChar());
         * }</pre>
         *
         * <p>Object counterpart of {@link #addChar} method. It is advised to use {@link #addChar} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        @Override
        default void forEachRemaining(Consumer<? super Character> action) {
            if (action instanceof CharConsumer) {
                forEachRemaining((CharConsumer) action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(getClass());
                }
                forEachRemaining((CharConsumer) action::accept);
            }
        }
    }
    
    /**
     * An array iterator specialized for {@code int} values.
     */
    public static interface OfInt extends ArrayIterator<Integer>, PrimitiveIterator.OfInt {

        /**
         * Returns the next {@code int} element in the iteration.
         *
         * @return the next {@code int} element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        int nextInt();

        /**
         * Returns the previous {@code int} element in the iteration.
         *
         * @return the previous {@code int} element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous element.
         */
        int previousInt();

        /**
         * Replaces the last element returned by {@link #nextInt} or {@link #previousInt} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addInt} have been
         * called after the last call to {@code nextInt} or {@code previousInt}.
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setInt} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         */
        void setInt(int value);

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #nextInt}, if any, and after the element that would be returned by
         * {@link #previousInt}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        void addInt(int value);

        /**
         * Returns the next element in the iteration.
         *
         * <p>Object counterpart of {@link #nextInt} method. It is advised to use {@link #nextInt} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        @Override
        default Integer next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return nextInt();
        }

        /**
         * Returns the previous element in the iteration.
         *
         * <p>Object counterpart of {@link #previousInt} method. It is advised to use {@link #previousInt} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous elements.
         */
        @Override
        default Integer previous() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return previousInt();
        }

        /**
         * Replaces the last element returned by {@link #nextInt} or {@link #previousInt} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addInt} have been
         * called after the last call to {@code next} or {@code previousInt}.
         *
         * <p>Object counterpart of {@link #setInt} method. It is advised to use {@link #setInt} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code set} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         * @throws NullPointerException if value is {@code null}.
         */
        @Override
        default void set(Integer value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            setInt(value);
        }

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #next}, if any, and after the element that would be returned by
         * {@link #previousInt}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * <p>Object counterpart of {@link #addInt} method. It is advised to use {@link #addInt} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        @Override
        default void add(Integer value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            addInt(value);
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextInt());
         * }</pre>
         *
         * <p>Object counterpart of {@link #addInt} method. It is advised to use {@link #addInt} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        @Override
        default void forEachRemaining(Consumer<? super Integer> action) {
            if (action instanceof IntConsumer) {
                forEachRemaining((IntConsumer) action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(getClass());
                }
                forEachRemaining((IntConsumer) action::accept);
            }
        }
    }

    /**
     * An array iterator specialized for {@code long} values.
     */
    public static interface OfLong extends ArrayIterator<Long>, PrimitiveIterator.OfLong {

        /**
         * Returns the next {@code long} element in the iteration.
         *
         * @return the next {@code long} element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        long nextLong();

        /**
         * Returns the previous {@code long} element in the iteration.
         *
         * @return the previous {@code long} element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous element.
         */
        long previousLong();

        /**
         * Replaces the last element returned by {@link #nextLong} or {@link #previousLong} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addLong} have been
         * called after the last call to {@code nextInt} or {@code previousInt}.
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setInt} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         */
        void setLong(long value);

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #nextLong}, if any, and after the element that would be returned by
         * {@link #previousLong}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        void addLong(long value);

        /**
         * Returns the next element in the iteration.
         *
         * <p>Object counterpart of {@link #nextLong} method. It is advised to use {@link #nextLong} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        @Override
        default Long next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return nextLong();
        }

        /**
         * Returns the previous element in the iteration.
         *
         * <p>Object counterpart of {@link #previousLong} method. It is advised to use {@link #previousLong} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous elements.
         */
        @Override
        default Long previous() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return previousLong();
        }

        /**
         * Replaces the last element returned by {@link #nextLong} or {@link #previousLong} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addLong} have been
         * called after the last call to {@code next} or {@code previousInt}.
         *
         * <p>Object counterpart of {@link #setLong} method. It is advised to use {@link #setLong} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setLong} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextLong} nor {@code previousLong} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         * @throws NullPointerException if value is {@code null}.
         */
        @Override
        default void set(Long value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            setLong(value);
        }

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #next}, if any, and after the element that would be returned by
         * {@link #previousLong}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * <p>Object counterpart of {@link #addLong} method. It is advised to use {@link #addLong} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        @Override
        default void add(Long value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            addLong(value);
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextLong());
         * }</pre>
         *
         * <p>Object counterpart of {@link #addLong} method. It is advised to use {@link #addLong} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        @Override
        default void forEachRemaining(Consumer<? super Long> action) {
            if (action instanceof LongConsumer) {
                forEachRemaining((LongConsumer) action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(getClass());
                }
                forEachRemaining((LongConsumer) action::accept);
            }
        }
    }

    /**
     * An array iterator specialized for {@code float} values.
     */
    public static interface OfFloat extends ArrayIterator<Float> {

        /**
         * Returns the next {@code float} element in the iteration.
         *
         * @return the next {@code float} element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        float nextFloat();

        /**
         * Returns the previous {@code float} element in the iteration.
         *
         * @return the previous {@code float} element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous element.
         */
        float previousFloat();

        /**
         * Replaces the last element returned by {@link #nextFloat} or {@link #previousFloat} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addFloat} have been
         * called after the last call to {@code nextInt} or {@code previousInt}.
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setInt} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         */
        void setFloat(float value);

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #nextFloat}, if any, and after the element that would be returned by
         * {@link #previousFloat}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        void addFloat(float value);

        /**
         * Returns the next element in the iteration.
         *
         * <p>Object counterpart of {@link #nextFloat} method. It is advised to use {@link #nextFloat} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        @Override
        default Float next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return nextFloat();
        }

        /**
         * Returns the previous element in the iteration.
         *
         * <p>Object counterpart of {@link #previousFloat} method. It is advised to use {@link #previousFloat} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous elements.
         */
        @Override
        default Float previous() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return previousFloat();
        }

        /**
         * Replaces the last element returned by {@link #nextFloat} or {@link #previousFloat} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addFloat} have been
         * called after the last call to {@code next} or {@code previousInt}.
         *
         * <p>Object counterpart of {@link #setFloat} method. It is advised to use {@link #setFloat} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setFloat} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextFloat} nor {@code previousFloat} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         * @throws NullPointerException if value is {@code null}.
         */
        @Override
        default void set(Float value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            setFloat(value);
        }

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #next}, if any, and after the element that would be returned by
         * {@link #previousFloat}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * <p>Object counterpart of {@link #addFloat} method. It is advised to use {@link #addFloat} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        @Override
        default void add(Float value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            addFloat(value);
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextFloat());
         * }</pre>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        default void forEachRemaining(FloatConsumer action) {
            while (hasNext()) {
                action.accept(nextFloat());
            }
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextFloat());
         * }</pre>
         *
         * <p>Object counterpart of {@link #addFloat} method. It is advised to use {@link #addFloat} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        @Override
        default void forEachRemaining(Consumer<? super Float> action) {
            if (action instanceof FloatConsumer) {
                forEachRemaining((FloatConsumer) action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(getClass());
                }
                forEachRemaining((FloatConsumer) action::accept);
            }
        }
    }

    /**
     * An array iterator specialized for {@code double} values.
     */
    public static interface OfDouble extends ArrayIterator<Double>, PrimitiveIterator.OfDouble {

        /**
         * Returns the next {@code double} element in the iteration.
         *
         * @return the next {@code double} element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        double nextDouble();

        /**
         * Returns the previous {@code double} element in the iteration.
         *
         * @return the previous {@code double} element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous element.
         */
        double previousDouble();

        /**
         * Replaces the last element returned by {@link #nextDouble} or {@link #previousDouble} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addDouble} have been
         * called after the last call to {@code nextInt} or {@code previousInt}.
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setInt} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextInt} nor {@code previousInt} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         */
        void setDouble(double value);

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #nextDouble}, if any, and after the element that would be returned by
         * {@link #previousDouble}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        void addDouble(double value);

        /**
         * Returns the next element in the iteration.
         *
         * <p>Object counterpart of {@link #nextDouble} method. It is advised to use {@link #nextDouble} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no more elements.
         */
        @Override
        default Double next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return nextDouble();
        }

        /**
         * Returns the previous element in the iteration.
         *
         * <p>Object counterpart of {@link #previousDouble} method. It is advised to use {@link #previousDouble} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @return next element in the iteration.
         * @throws NoSuchElementException if the iteration has no previous elements.
         */
        @Override
        default Double previous() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            return previousDouble();
        }

        /**
         * Replaces the last element returned by {@link #nextDouble} or {@link #previousDouble} with the specified element
         * (optional operation). This call can be made only if neither {@link #remove} nor {@link #addDouble} have been
         * called after the last call to {@code next} or {@code previousInt}.
         *
         * <p>Object counterpart of {@link #setDouble} method. It is advised to use {@link #setDouble} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value with which to replace the last element returned by {@code nextInt} or
         * {@code previousInt}.
         * @throws UnsupportedOperationException if the {@code setDouble} operation is not supported by this array iterator.
         * @throws IllegalStateException if neither {@code nextDouble} nor {@code previousDouble} have been called, or
         * {@code remove} or {@code addInt} have been called after the last call to {@code nextInt} or
         * {@code previousInt}.
         * @throws NullPointerException if value is {@code null}.
         */
        @Override
        default void set(Double value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            setDouble(value);
        }

        /**
         * Inserts the specified element into the array (optional operation). The element is inserted immediately before
         * the element that would be returned by {@link #next}, if any, and after the element that would be returned by
         * {@link #previousDouble}, if any.  (If the array contains no elements, the new element becomes the sole element
         * on the array.) The new element is inserted before the implicit cursor: a subsequent call to {@code nextInt}
         * would be unaffected, and a subsequent call to {@code previousInt} would return the new element. (This call
         * increases by one the value that would be returned by a call to {@code nextIndex} or {@code previousIndex}.)
         *
         * <p>Object counterpart of {@link #addDouble} method. It is advised to use {@link #addDouble} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param value the value to insert.
         * @throws UnsupportedOperationException if the {@code add} method is not supported by this list iterator.
         */
        @Override
        default void add(Double value) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(getClass());
            }
            addDouble(value);
        }

        /**
         * Performs the given action for each remaining element, in the order elements occur when iterating, until all
         * elements have been processed or the action throws an exception. Errors or runtime exceptions thrown by the
         * action are relayed to the caller.
         *
         * <p>The default implementation behaves as if:
         * <pre>{@code
         *     while (hasNext())
         *         action.accept(nextDouble());
         * }</pre>
         *
         * <p>Object counterpart of {@link #addDouble} method. It is advised to use {@link #addDouble} instead to prevent
         * unnecessary boxing of primitives.</p>
         *
         * @param action the action to be performed for each element.
         * @throws NullPointerException if the specified action is null.
         */
        @Override
        default void forEachRemaining(Consumer<? super Double> action) {
            if (action instanceof DoubleConsumer) {
                forEachRemaining((DoubleConsumer) action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(getClass());
                }
                forEachRemaining((DoubleConsumer) action::accept);
            }
        }
    }
}
